@@ -28,7 +28,6 @@ import tornado.tcpclient
 import tornado.gen
 
 import asyncmc
-#import memcache
 import socket
 import datetime
 import random
@@ -93,8 +92,7 @@ class DataRouter(tornado.web.Application):
             self, idle_connection_timeout=90)
         http_server.listen(port)
 
-        # --- start the memcache client
-#        self.cache = memcache.Client(servers=self.mc_servers)
+        # --- start the memcache-client
         self.cache = asyncmc.Client(
             servers=self.mc_servers, loop=tornado.ioloop.IOLoop.current())
 
@@ -113,20 +111,17 @@ class DataRouter(tornado.web.Application):
         self.logger.info("shutting down router")
 
     # -------------------------------------------------------------------------
-    @tornado.gen.coroutine
-    def with_timeout(self, future):
-        generator = yield tornado.gen.with_timeout(self.timeout, future)
-        return generator
+    async def with_timeout(self, future):
+        return await tornado.gen.with_timeout(self.timeout, future)
 
     # -------------------------------------------------------------------------
-    @tornado.gen.coroutine
-    def fetch(self, req, addr, port):
+    async def fetch(self, req, addr, port):
         client = tornado.tcpclient.TCPClient()
-        stream = yield self.with_timeout(client.connect(addr, port))
+        stream = await self.with_timeout(client.connect(addr, port))
 
         try:
-            yield self.with_timeout(stream.write(encode_message(req)))
-            response = yield self.with_timeout(stream.read_until(b"\n"))
+            await self.with_timeout(stream.write(encode_message(req)))
+            response = await self.with_timeout(stream.read_until(b"\n"))
         finally:
             try:
                 stream.close()
@@ -136,17 +131,15 @@ class DataRouter(tornado.web.Application):
         return response
 
     # -------------------------------------------------------------------------
-    @tornado.gen.coroutine
-    def process_request(self, req, real_time):
+    async def process_request(self, req, real_time):
         # --- determine the unique request id, used for caching
         req_uid = request_to_uid(req, real_time)
 
         # --- first try fetching response from cache
         while True:
-#            response = self.cache.get(req_uid)
-            response = yield self.cache.get(req_uid)
+            response = await self.cache.get(req_uid)
             if response == "CacheLocked":
-                yield tornado.gen.sleep(0.25)
+                await tornado.gen.sleep(0.25)
             else:
                 break
 
@@ -155,20 +148,22 @@ class DataRouter(tornado.web.Application):
             #     router.
             #     NB: we first lock the cache with a timeout and then we only
             #         set the cache if the response is valid.
-#            self.cache.set(req_uid, "CacheLocked",
-#                           self.timeout.seconds, noreply=True)
-            yield self.cache.set(req_uid, "CacheLocked",
-                                 self.timeout.seconds, noreply=True)
+            await self.cache.set(
+                req_uid, "CacheLocked", self.timeout.seconds, noreply=True)
 
             while True:
                 # --- select dataserver or return error if none is available
                 try:
                     addr, port = self.select_dataserver()
                 except NoDataserverAvailable:
-                    return 503, "Dataservers not available or unresponsive"
+                    resp = {
+                        "type": "NoDataserverAvailable",
+                        "payload": "Dataservers not available or unresponsive",
+                    }
+                    return 503, encode_message(resp)
 
                 try:
-                    response = yield self.fetch(req, addr, port)
+                    response = await self.fetch(req, addr, port)
                 except (tornado.gen.TimeoutError,
                         tornado.iostream.StreamClosedError) as err:
                     # --- connection unresponsive: drop dataserver from list
@@ -183,15 +178,15 @@ class DataRouter(tornado.web.Application):
             else:
                 expiry = int(Date.today().eod().timestamp())
 
-#            self.cache.set(req_uid, response, expiry, noreply=True)
-            yield self.cache.set(req_uid, response, expiry, noreply=True)
+            await self.cache.set(req_uid, response, expiry, noreply=True)
 
-            self.logger.debug("returning value from dataserver "
+            self.logger.debug(
+                "returning value from dataserver "
                 "(rt={0!s}), req_uid='{1!s}'".format(real_time, req_uid))
 
         else:
-            self.logger.debug("returning value "
-                "from cache (rt={0!s})".format(real_time))
+            self.logger.debug(
+                "returning value from cache (rt={0!s})".format(real_time))
 
         return 200, response
 
@@ -215,9 +210,7 @@ class RegistrationHandler(tornado.web.RequestHandler):
 ###############################################################################
 class BbgBDPHandler(tornado.web.RequestHandler):
     # -------------------------------------------------------------------------
-    @tornado.web.asynchronous
-    @tornado.gen.coroutine
-    def get(self):
+    async def get(self):
         self.set_header("Content-Type", "application/json")
         self.set_header("Access-Control-Allow-Origin", "*")
 
@@ -230,18 +223,16 @@ class BbgBDPHandler(tornado.web.RequestHandler):
 
         rt = self.get_argument("rt", False)
 
-        status, resp = yield self.application.process_request(request, rt)
+        status, resp = await self.application.process_request(request, rt)
 
-        self.set_status(status, None if status == 200 else resp)
+        self.set_status(status)
         self.write(resp)
 
 
 ###############################################################################
 class BbgBDHHandler(tornado.web.RequestHandler):
     # -------------------------------------------------------------------------
-    @tornado.web.asynchronous
-    @tornado.gen.coroutine
-    def get(self):
+    async def get(self):
         self.set_header("Content-Type", "application/json")
         self.set_header("Access-Control-Allow-Origin", "*")
 
@@ -257,7 +248,7 @@ class BbgBDHHandler(tornado.web.RequestHandler):
 
         rt = self.get_argument("rt", False)
 
-        status, resp = yield self.application.process_request(request, rt)
+        status, resp = await self.application.process_request(request, rt)
 
         self.set_status(status)
         self.write(resp)
@@ -266,8 +257,6 @@ class BbgBDHHandler(tornado.web.RequestHandler):
 ###############################################################################
 class BbgUniqueIdHandler(tornado.web.RequestHandler):
     # -------------------------------------------------------------------------
-    @tornado.web.asynchronous
-    @tornado.gen.coroutine
     def get(self):
         self.set_header("Content-Type", "application/json")
         self.set_header("Access-Control-Allow-Origin", "*")
